use std::fmt::Display;

use binrw::{BinRead, BinWrite};

use crate::{array_of::ArrayOf, types::Point};

use super::ObjectBase;

#[derive(BinRead, BinWrite, Debug)]
pub enum LineType {
    #[brw(magic = 0x0001u16)]
    Point { _zero: u32, point: Point },
    #[brw(magic = 0x0100u16)]
    Line { _zero: u32, points: ArrayOf<Point> },
    #[brw(magic = 0x0300u16)]
    Bezier { _zero: u32, points: ArrayOf<Point> },
}

#[derive(BinRead, BinWrite, Debug)]
pub struct Lines {
    pub base: ObjectBase,
    pub lines: ArrayOf<LineType, u64>,
}

impl Display for Lines {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{} ", self.base)?;
        for line in self.lines.iter() {
            match line {
                LineType::Point { _zero, point } => write!(f, "point: {}", point)?,
                LineType::Line { _zero, points } => {
                    write!(f, "points: ")?;
                    points.iter().try_for_each(|x| write!(f, "{}", x))?;
                }
                LineType::Bezier { _zero, points } => {
                    write!(f, "points: ")?;
                    points.iter().try_for_each(|x| write!(f, "{}", x))?;
                }
            }
        }
        Ok(())
    }
}
